package org.ow2.orchestra.console.GWTConsole.client.ListInstance;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.cellview.client.ColumnSortList;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.view.client.AsyncDataProvider;
import com.google.gwt.view.client.HasData;
import com.google.gwt.view.client.Range;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsoleCellTable;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @author Loic Albertin
 */
public class InstancesDataProvider extends AsyncDataProvider<MInstance> {

  private String processId;
  private String instanceId;

  private OrchestraManagementServicesAsync orchestraManagement = GWT.create(OrchestraManagementServices.class);

  private final ConsoleCellTable<MInstance> table;

  private final PTabLayoutPanel tabLayoutPanel;

  private final HashMap<Column<?, ?>, String> sortableColumns = new HashMap<Column<?, ?>, String>();

  public InstancesDataProvider(String processId, String instanceId,
                               ConsoleCellTable<MInstance> table, HashMap<Column<?, ?>,
                  String> sortableColumns, PTabLayoutPanel tabLayoutPanel) {
    super();
    this.processId = processId;
    this.instanceId = instanceId;
    this.table = table;
    this.tabLayoutPanel = tabLayoutPanel;
    this.sortableColumns.putAll(sortableColumns);
  }

  @Override
  protected void onRangeChanged(HasData<MInstance> display) {
    if (this.instanceId != null) {
      //Selection of a specific instance
      orchestraManagement.getInstance(
              this.instanceId, new AsyncCallback<MInstance>() {

                public void onSuccess(final MInstance instance) {
                  // check that the result list contains 1 element
                  if (instance != null) {
                    List<MInstance> mInstanceList = new ArrayList<MInstance>();
                    mInstanceList.add(instance);
                    updateRowCount(1, true);
                    updateRowData(0, mInstanceList);
                  } else {
                    updateRowCount(0, true);
                    updateRowData(0, new ArrayList<MInstance>());
                  }
                  ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
                }

                public void onFailure(final Throwable caught) {
                  if (InstancesDataProvider.this.tabLayoutPanel
                          .tabAlreadyExists(InstancesDataProvider.this.instanceId, PTab.Type.LIST_INSTANCE_ACTIVITY) != -1) {
                    GWT.log("InstanceDetailsGrid - refreshCurrentList : onFailure : " + caught.getMessage());
                    ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                    if (caught instanceof OrchestraManagementException) {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                              ((OrchestraManagementException) caught).getDetails());
                    } else {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
                    }
                  }
                }
              });
    } else {
      //Selection on range for all instances
      final Range range = display.getVisibleRange();
      // Get the ColumnSortInfo from the table.
      final ColumnSortList sortList = this.table.getColumnSortList();

      String sortedColumn = "";
      boolean isAscending = true;

      if (sortList.size() > 0 && this.sortableColumns.containsKey(sortList.get(0).getColumn())) {
        ColumnSortList.ColumnSortInfo sortInfo = sortList.get(0);
        isAscending = sortInfo.isAscending();
        sortedColumn = this.sortableColumns.get(sortList.get(0).getColumn());
      }

      orchestraManagement.getInstances(
              range.getStart(), range.getLength(), sortedColumn, isAscending, this.processId,
              new AsyncCallback<MInstancePage>() {

                public void onSuccess(final MInstancePage mInstancePage) {
                  updateRowCount((int) mInstancePage.getTotalInstancesCount(), true);
                  updateRowData(range.getStart(), mInstancePage.getInstances());
                  ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
                }

                public void onFailure(final Throwable caught) {
                  GWT.log("InstanceGrid - refreshCurrentList: onFailure : " + caught.getMessage());
                  if (caught instanceof OrchestraManagementException) {
                    ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                    PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                            ((OrchestraManagementException) caught).getDetails());
                  } else {
                    PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
                  }
                }
              });
    }
  }
}
